/*
** Module   : BaseDataType.java
** Abstract : Progress 4GL compatible operator logic
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ---------------------------Description----------------------------
** 001 GES 20050518   @21222 Created initial version with full support for all Progress logical
**                           operators and common functions.
** 002 GES 20050606   @21423 Minor additions for function support.
** 003 ECF 20050629   @21601 Implement Serializable. Will need this to push wrapper objects
**                           over the wire.
** 004 SVG 20050720   @21762 Cloning support added.
** 005 GES 20050815   @22117 Added Undoable interface support.
** 006 GES 20050825   @22252 Added getTypeName().
** 007 ECF 20051005   @22966 Introduced an abstract hashCode() method. This forces subclasses
**                           to provide an implementation which is consistent with this class'
**                           implementation of the equals() method.
** 008 GES 20060208   @24385 Added defaultFormatString().
** 009 GES 20060301   @24840 Added generateUnknown() static factory method.
** 010 NVS 20060407   @25407 Added isUnknownValue() and notUnknownValue() methods.
** 011 GES 20060417   @25551 Added default value generation.
** 012 GES 20060724   @28156 Added a forced assign() abstract method.
** 013 GES 20060807   @28469 Moved to externalizable interface to optimize network performance.
** 014 GES 20061109   @31049 Added formatLength().
** 015 ECF 20070720   @34626 Added safety check to equals(). Check type of object before casting
**                           it to BaseDataType to prevent ClassCastException.
** 016 SIY 20100812          Removed unnecessary cast.
** 017 CA  20130119          Added castLogical and castBoolean, which cast the current value
**                           using DYNAMIC-FUNCTION's rules.
** 018 GES 20130128          Clarified some javadoc.
** 019 GES 20130312          Removed castLogical() and castBoolean(). This same functionality is
**                           implemented in the logical(BDT) c'tor.
** 020 EVK 20131130          Added elementsOfType static method.
*/

package com.goldencode.p2j.util;

import java.io.*;

/**
 * An abstract class that implements a Progress 4GL compatible data type
 * which has common logical operator processing.  Meant to be the parent
 * class for specific Progress compatible data type wrappers.
 * <p>
 * Each subclass should implement <code>Comparable</code> but is only
 * required to implement <code>compareTo()</code> as the <code>equals()</code>
 * method is implemented in this class.
 * <p>
 * All processing relies upon the implementation of the following methods
 * in each concrete subclass:
 * <p>
 * <pre>
 * boolean      isUnknown()
 * void         setUnknown(boolean)
 * int          compareTo(Object) - this comes from the Comparable interface
 * BaseDataType instantiateUnknown()
 * BaseDataType duplicate()
 * void         assign(Undoable) - this comes from the Undoable interface
 * void         assign(BaseDataType, boolean)
 * String       toString()
 * String       toStringMessage()
 * String       toStringExport()
 * String       defaultFormatString()
 * </pre>
 *
 * @author    GES
 */
public abstract class BaseDataType
implements Comparable,
           Externalizable,
           Undoable
{
   /**
    * Default constructor (only used for de-serialization).
    */
   public BaseDataType()
   {
   }
   
   /**
    * Creates a new instance of the given type that represents the 
    * <code>unknown value</code>.
    *
    * @return   An instance that represents the <code>unknown value</code>.
    */
   public static BaseDataType generateUnknown(Class cls)   
   {
      BaseDataType bdt = null;
      
      try
      {
         // use the default constructor
         bdt = (BaseDataType) cls.newInstance();
      }
      
      catch (Exception expt)
      {
         // what else to do here (the calling code will probably fail with
         // a class cast exception)?
         return new unknown();
      }
      
      return bdt.instantiateUnknown();
   }
   
   /**
    * Creates a new instance of the given type that represents the 
    * default initialized value for that type.
    *
    * @return   An instance that represents the default value.
    */
   public static BaseDataType generateDefault(Class cls)   
   {
      BaseDataType bdt = null;
      
      try
      {
         // use the default constructor
         bdt = (BaseDataType) cls.newInstance();
      }
      
      catch (Exception expt)
      {
         throw new RuntimeException(expt);
      }
      
      return bdt.instantiateDefault();
   }
   
   /**
    * Calculate the length of a formatted value (the text form of the value)
    * using this format string.
    *
    * @param    fmt
    *           The format string.
    *
    * @return   The length in characters of the any resulting formatted
    *           value.
    */
   public static int calcFormatLength(String fmt, Class cls)
   {
      BaseDataType bdt = null;
      
      try
      {
         // use the default constructor
         bdt = (BaseDataType) cls.newInstance();
      }
      
      catch (Exception expt)
      {
         throw new RuntimeException(expt);
      }
      
      return bdt.formatLength(fmt);
   }

   /**
    * Check list of BaseType of arguments and return {@code true} if all BaseTypes are specific
    * type.
    *
    * @param    cls
    *           Specific BaseDataType class witch will be checked to the arguments.
    * @param    args
    *           list of BaseType instances.
    *
    * @return   The {@code true} if all BaseTypes are character.
    */
   public static boolean elementsOfType(Class<? extends BaseDataType> cls, BaseDataType... args)
   {
      if (args.length == 0)
      {
         return true;
      }

      for (BaseDataType arg : args)
      {

         if (!cls.isInstance(arg))
         {
            return false;
         }
      }
      return true;
   }
   
   /**
    * Calculate the length of a formatted value (the text form of the value)
    * using this format string.
    *
    * @param    fmt
    *           The format string.
    *
    * @return   The length in characters of the any resulting formatted
    *           value.
    */
   public int formatLength(String fmt)
   {
      if (fmt == null)
      {
         fmt = defaultFormatString();
      }
      
      return fmt.length();
   }
   
   /**
    * Determines if this instance and the given instance are equivalent.
    * Only an object of type <code>BaseDataType</code> is compared with this
    * instance;  objects of other types cause this test to return
    * <code>false</code>.
    *
    * @param    obj
    *           The instance to compare against.
    *
    * @return   <code>true</code> if the objects compare equivalently;
    *           <code>false</code> if they do not, or if the parameter is not
    *           a <code>BaseDataType</code> instance.
    */
   public boolean equals(Object obj)   
   {
      if (!(obj instanceof BaseDataType))
      {
         return false;
      }
      
      return CompareOps.equals(this, (BaseDataType) obj);
   }
   
   /**
    * The default implementation of this method provided by
    * <code>Object</code> is overridden here with an abstract method to force
    * subclasses to provide a concrete implementation which is consistent with
    * this class' implementation of {@link #equals}.
    *
    * @return  The hash code for this object instance.
    */
   public abstract int hashCode();
   
   /**
    * Returns the largest instance.
    *
    * @param    b
    *           The instance to compare against.
    *
    * @return   The largest instance.
    */
   public BaseDataType maximum(BaseDataType b)   
   {
      if (isUnknown() || b.isUnknown())
         return instantiateUnknown();
      
      return (CompareOps.greaterThan(this, b)) ? this : b;
   }
      
   /**
    * Returns the smallest instance.
    *
    * @param    b
    *           The instance to compare against.
    *
    * @return   The smallest instance.
    */
   public BaseDataType minimum(BaseDataType b)   
   {
      if (isUnknown() || b.isUnknown())
         return instantiateUnknown();
      
      return (CompareOps.lessThan(this, b)) ? this : b;
   }
   
   /**
    * This is a form of a copy constructor that makes a deep copy of the
    * current instance and returns this copy as a new instance of the same
    * class.
    *
    * @return   An instance of the same class whose data is identical to
    *           that of this instance.
    */
   public Undoable deepCopy()   
   {
      return (Undoable) duplicate();
   }
   
   /**
    * Returns the legacy type name of the sub-class as the type was known in
    * the 4GL.  Generally, this is equivalent to an uppercased version of the
    * base name of the class file (all package information is removed). Child
    * classes can override this default behavior.
    * 
    * @return   The legacy type name.
    */
   public String getTypeName()
   {
      String cls = this.getClass().getName();
      
      int idx = cls.lastIndexOf('.') + 1;
      
      return cls.substring(idx).toUpperCase();
   }

   /**
    * Reports if this instance represents the <code>unknown value</code>.
    *
    * @return   <code>true</code> if this instance is set to the
    *           <code>unknown value</code>.
    */
   public logical isUnknownValue()
   {
      return new logical(isUnknown());
   }
   
   /**
    * Reports if this instance represents some known value.
    *
    * @return   <code>true</code> if this instance is not set to the
    *           <code>unknown value</code>.
    */
   public logical notUnknownValue()
   {
      return new logical(!isUnknown());
   }
   
   /**
    * Creates a new instance of the same type that represents the 
    * default initialized value.
    *
    * @return   An instance that represents the default value.
    */
   public BaseDataType instantiateDefault()   
   {
      BaseDataType bdt = null;
      
      try
      {
         // use the default constructor
         bdt = this.getClass().newInstance();
      }
      
      catch (Exception expt)
      {
         throw new RuntimeException(expt);
      }
      
      return bdt;
   }
   
   /**
    * Reports if this instance represents the <code>unknown value</code>.
    *
    * @return   <code>true</code> if this instance is set to the
    *           <code>unknown value</code>.
    */
   public abstract boolean isUnknown();
   
   /**
    * Sets the state of this instance's <code>unknown value</code> flag or
    * state to <code>true</code>.
    * <p>
    * <b>Warning: the data stored in this instance may be invalid after
    * calling this method.</b>
    */
   public abstract void setUnknown();
   
   /**
    * Sets the state (data and unknown value) of this instance based on the
    * state of the passed instance.
    *
    * @param    value
    *           The instance from which to copy state.
    * @param    force
    *           Flag to indicate if a pending error should be ignored (see 
    *           {@link ErrorManager#isPendingError} and the assignment
    *           forced regardless.  Note that normal assignment silently
    *           exits with no change if there is a pending error. If this
    *           flag is <code>true</code>, that check is bypassed and the
    *           assignment will always be done.
    */
   public abstract void assign(BaseDataType value, boolean force);
   
   /**
    * Does the same as standard <code>clone()</code> method but returns an
    * instance of <code>BaseDataType</code> and doesn't throw the
    * <code>CloneNotSupportedException</code>.
    *
    * @return   A clone of this instance.
    */
   public abstract BaseDataType duplicate();

   /**
    * Creates a new instance of the same type that represents the 
    * <code>unknown value</code>.
    *
    * @return   An instance that represents the <code>unknown value</code>.
    */
   public abstract BaseDataType instantiateUnknown();   
   
   /**
    * Creates a string representation of the instance data using the given
    * format string.  If the instance represents the
    *  <code>unknown value</code>, a '?' will be returned.
    *
    * @param    fmt
    *           The format string to use.
    *
    * @return   The formatted string.
    */
   public abstract String toString(String fmt);   
   
   /**
    * Creates a string representation of the instance data in a form that is
    * compatible with the <code>MESSAGE</code> language statement.  If the
    * instance represents the <code>unknown value</code>, a '?' will be
    * returned.
    *
    * @return   The 'message' formatted string.
    */
   public abstract String toStringMessage();   
   
   /**
    * Creates a string representation of the instance data using the 'export'
    * format.  If the instance represents the <code>unknown value</code>, a
    * '?' will be returned.
    *
    * @return   The 'export' formatted string.
    */
   public abstract String toStringExport();      
   
   /**
    * Return the default display format string for this type.
    *
    * @return   The default format string.
    */
   public abstract String defaultFormatString();
}
